<?php
/**
 * Joom!Fish - Multi Lingual extention and translation manager for Joomla!
 * Copyright (C) 2003-2008 Think Network GmbH, Munich
 * 
 * All rights reserved.  The Joom!Fish project is a set of extentions for 
 * the content management system Joomla!. It enables Joomla! 
 * to manage multi lingual sites especially in all dynamic information 
 * which are stored in the database.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * The "GNU Lesser General Public License" (LGPL) is available at
 * http: *www.gnu.org/copyleft/lgpl.html
 * -----------------------------------------------------------------------------
 * $Id: ReadMe,v 1.2 2005/03/15 11:07:01 akede Exp $
 * @package joomfish
 * @subpackage system.jfdatabase_bot
 * @version 2.0
 *
*/

/** ensure this file is being included by a parent file */
defined( '_JEXEC' ) or die( 'Restricted access' );

// In PHP5 this should be a instance_of check
// Currently Joom!Fish does not need to be active in Administrator
// This might be an extended version
if($mainframe->isAdmin()) {
	return;
}
// Joom!Fish bot get's only activated if essential files are missing
//if ( !file_exists( JPATH_PLUGINS .DS. 'system' .DS. 'jfdatabase' .DS. 'jfdatabase.class.php' )) {
if ( !file_exists( dirname(__FILE__) .DS. 'jfdatabase' .DS. 'jfdatabase_inherit.php' )) {
	JError::raiseNotice('no_jf_plugin', JText::_('Joom!Fish plugin not installed correctly. Plugin not executed'));
	return;
}
// experiment in routing based on sef language code
//JRouter::attachBuildRule and attachParseRule
// Get the router
$app	= &JFactory::getApplication();
$router = &$app->getRouter();
$router->attachBuildRule("routeJFRule");
$router->attachParseRule("parseJFRule");


/**
* Exchange of the database abstraction layer for multi lingual translations.
*/
class plgSystemJFDatabase extends JPlugin{
	/**
	 * stored configuration from plugin
	 *
	 * @var object configuration information
	 */
	var $_config = null;

	function plgSystemJFDatabase(& $subject, $config)
	{
		global $mainframe;
		if ($mainframe->isAdmin()) {
			// This plugin is only relevant for use within the frontend!
			return;
		}
		parent::__construct($subject, $config);

		// put params in registry so I have easy access to them later
		$conf =& JFactory::getConfig();
		$conf->setValue("jfdatabase.params",$this->params);

		$this->_config = array(
		'adapter' 	=> $this->params->get('dbadapter', "inheritor")
		);

		// Must do this here in case other plugins instantiate language!
		if(JPath::find(JPATH_ADMINISTRATOR .DS. 'components' .DS. 'com_joomfish','JoomfishManager.class.php')) {
			require_once( JPATH_ADMINISTRATOR .DS. 'components' .DS. 'com_joomfish' .DS.'JoomfishManager.class.php' );
			$this->_jfInitialize();
			//$this->_discoverJFLanguage();
		} else {
			JError::raiseNotice('no_jf_component', JText::_('Joom!Fish component not installed correctly. Plugin not executed'));
		}
	}

	/**
	 * During this event we setup the database and link it to the Joomla! ressources for future use
	 * @return void
	 */
	function onAfterInitialise()
	{
		global $mainframe;
		if ($mainframe->isAdmin()) {
			// This plugin is only relevant for use within the frontend!
			return;
		}
		$this->_setupJFDatabase();
	}

	/**
	 * We determin the language from the information got by the routing plugins or the browser
	 * @return void
	 */
	function onAfterRoute()
	{

		global $mainframe;
		if ($mainframe->isAdmin()) {
			// This plugin is only relevant for use within the frontend!
			return;
		}
		$this->_discoverJFLanguage();
	}

	/**
	 * Setup for the Joom!Fish database connectors, overwriting the original instances of Joomla!
	 * Which connector is used and which technique is based on the extension configuration
	 * @return void
	 */
	function _setupJFDatabase(){
		if ($this->_config["adapter"] == "decorator") {
			if (file_exists( JPATH_ADMINISTRATOR .DS. 'components' .DS. 'com_joomfish' .DS. 'jfdatabase_decorator.class.php' )) {
				require_once( JPATH_ADMINISTRATOR .DS. 'components' .DS. 'com_joomfish' .DS. 'jfdatabase_decorator.class.php' );

				$db = & JFactory::getDBO();
				$db = new JFDatabase();


				$conf =& JFactory::getConfig();
				$conf->setValue('config.mbf_content', 1 );
				$conf->setValue('config.multilingual_support', 1 );

				// TODO: check on legacy mode on or off
				$GLOBALS['database'] = $db;
			}
		}
		else {
			if (file_exists( dirname(__FILE__).DS.'jfdatabase'.DS.'jfdatabase_inherit.php' )) {
				require_once( dirname(__FILE__).DS.'jfdatabase'.DS.'jfdatabase_inherit.php' );

				$conf =& JFactory::getConfig();

				$host 		= $conf->getValue('config.host');
				$user 		= $conf->getValue('config.user');
				$password 	= $conf->getValue('config.password');
				$db   		= $conf->getValue('config.db');
				$dbprefix 	= $conf->getValue('config.dbprefix');
				$dbtype 	= $conf->getValue('config.dbtype');
				$debug 		= $conf->getValue('config.debug');
				$driver 	= $conf->getValue('config.dbtype');

				$options = array("driver"=>$driver, "host"=>$host, "user"=>$user, "password"=>$password, "database"=>$db, "prefix"=>$dbprefix,"select"=>true);

				$db = & JFactory::getDBO();
				$db = new JFDatabase($options);

				if ($db->getErrorNum() > 2) {
					JError::raiseError('joomla.library:'.$db->getErrorNum(), 'JDatabase::getInstance: Could not connect to database <br/>' . $db->getErrorMsg() );
				}

				$conf->setValue('config.mbf_content', 1 );
				$conf->setValue('config.multilingual_support', 1 );

				// legacy mode only
				// check on legacy mode on/off by testing existence of $database global
				if (array_key_exists('database',$GLOBALS)){
					$GLOBALS['database'] = new mldatabase($options);
					$GLOBALS['database']->debug($conf->getValue('config.debug'));
				}

				//echo phpinfo();
			}
		}

	}


	function _discoverJFLanguage ( ) {
		static $discovered;
		if (isset($discovered) && $discovered){
			return;
		}
		$discovered=true;
		
		$registry =& JFactory::getConfig();

		$lang =& JFactory::getLanguage();
		$mosConfig_lang          = $lang->getTag();
		$GLOBALS['mosConfig_defaultLang'] = $mosConfig_lang;        // Save the default language of the site

		// get params from registry in case function called statically
		$params = $registry->getValue("jfdatabase.params");

		$registry->setValue("config.defaultlang", $mosConfig_lang);

		$determitLanguage 		= $params->get( 'determitLanguage', 1 );
		$newVisitorAction		= $params->get( 'newVisitorAction', 'browser' );
		$use302redirect			= $params->get( 'use302redirect', 0 );
		$enableCookie			= $params->get( 'enableCookie', 1 );

		// get instance of JoomFishManager to obtain active language list and config values
		$jfm =&  JoomFishManager::getInstance();

		$client_lang = '';
		$lang_known = false;
		$jfcookie = JRequest::getVar('jfcookie', null ,"COOKIE");
		if (isset($jfcookie["lang"]) && $jfcookie["lang"] != "") {
			$client_lang = $jfcookie["lang"];
			$lang_known = true;
		}

		$uri =& JURI::getInstance();
		if ($lang = JRequest::getVar('lang', null ,"REQUEST")){
			if( $lang != '' ) {
				$client_lang = $lang;
				$lang_known = true;
			}
		}

		/*		if ( $client_lang != '' && ($lang = $uri->getVar("lang","")) ) {
		if( $lang != '' ) {
		$client_lang = $lang;
		}
		// no language chooses - assume from browser configuration
		}*/

		if ( !$lang_known && $determitLanguage &&
		key_exists( 'HTTP_ACCEPT_LANGUAGE', $_SERVER ) && !empty($_SERVER['HTTP_ACCEPT_LANGUAGE']) ) {

			switch ($newVisitorAction) {
				// usesing the first defined Joom!Fish language
				case 'joomfish':
					$activeLanguages = $jfm->getActiveLanguages();
					$client_lang = $activeLanguages[0]->getLanguageCode();
					break;

				case 'site':
					$jfLang = TableJFLanguage::createByJoomla( $mosConfig_lang );
					$client_lang = $jfLang->getLanguageCode();
					break;

					// no language chooses - assume from browser configuration
				case 'browser':
				default:
					// language negotiation by Kochin Chang, June 16, 2004
					// retrieve active languages from database
					$active_iso = array();
					$active_isocountry = array();
					$active_code = array();
					$activeLanguages = $jfm->getActiveLanguages();
					if( count( $activeLanguages ) == 0 ) {
						return $mosConfig_lang;
					}

					foreach ($activeLanguages as $lang) {
						$active_iso[] = $lang->iso;
						if( eregi('[_-]', $lang->iso) ) {
							$isocountry = split('[_-]',$lang->iso);
							$active_isocountry[] = $isocountry[0];
						}
						$active_code[] = $lang->shortcode;
					}

					// figure out which language to use - browser languages are based on ISO codes
					$browserLang = explode(',', $_SERVER["HTTP_ACCEPT_LANGUAGE"]);

					foreach( $browserLang as $lang ) {
						if( in_array($lang, $active_iso) ) {
							$client_lang = $lang;
							break;
						}
						$shortLang = substr( $lang, 0, 2 );
						if( in_array($shortLang, $active_isocountry) ) {
							$client_lang = $shortLang;
							break;
						}

						// compare with code
						if ( in_array($shortLang, $active_code) ) {
							$client_lang = $shortLang;
							break;
						}
					}
					break;
			}
		}

		// get the name of the language file for joomla
		$jfLang = TableJFLanguage::createByShortcode($client_lang, false);
		if( $jfLang === null && $client_lang!="") {
			$jfLang = TableJFLanguage::createByISO( $client_lang, false );
		}		
		else if( $jfLang === null) {
				$jfLang = TableJFLanguage::createByJoomla( $mosConfig_lang );
		}

		if( !$lang_known && $use302redirect ) {
			// using a 302 redirect means that we do not change the language on the fly the first time, but with a clean reload of the page

			$href= "index.php";
			$hrefVars = '';
			$queryString = JRequest::getVar('QUERY_STRING', null ,"_SERVER");
			if( !empty($queryString) ) {
				$vars = explode( "&", $queryString );
				if( count($vars) > 0 && $queryString) {
					foreach ($vars as $var) {
						if( eregi('=', $var ) ) {
							list($key, $value) = explode( "=", $var);
							if( $key != "lang" ) {
								if( $hrefVars != "" ) {
									$hrefVars .= "&amp;";
								}
								// ignore mosmsg to ensure it is visible in frontend
								if( $key != 'mosmsg' ) {
									$hrefVars .= "$key=$value";
								}
							}
						}
					}
				}
			}

			// Add the existing variables
			if( $hrefVars != "" ) {
				$href .= '?' .$hrefVars;
			}

			if( $jfLang->getLanguageCode() != null ) {
				$lang = 'lang=' .$jfLang->getLanguageCode();
			} else {
				// it's important that we add at least the basic parameter - as of the sef is adding the actual otherwise!
				$lang = 'lang=';
			}

			// if there are other vars we need to add a & otherwiese ?
			if( $hrefVars == '' ) {
				$href .= '?' . $lang;
			} else {
				$href .= '&amp;' . $lang;
			}
			$href = JRoute::_($href);
			
			$GLOBALS['iso_client_lang'] = $jfLang->getLanguageCode();
			header( 'HTTP/1.1 303 See Other' );
			header( "Location: ". $href );
			exit();
		}

		if( isset($jfLang) && $jfLang->code != "" && ($jfLang->active  || $jfm->getCfg("frontEndPreview") )) {
			$mosConfig_lang = $jfLang->code;
		} else {
			$jfLang = TableJFLanguage::createByJoomla( $mosConfig_lang );
			if( !$jfLang->active ) {
			?>
			<div style="background-color: #c00; color: #fff">
				<p style="font-size: 1.5em; font-weight: bold; padding: 10px 0px 10px 0px; text-align: center; font-family: Arial, Helvetica, sans-serif;">
				Joom!Fish config error: Default language is inactive!<br />&nbsp;<br />
				Please check configuration, try to use first active language</p>
			</div>
			<?php
			$activeLanguages = $jfm->getActiveLanguages();
			if( count($activeLanguages) > 0 ) {
				$jfLang = $activeLanguages[0];
				$mosConfig_lang = $jfLang->code;
			}
			else {
				// No active language defined - using system default is only alternative!
			}
			}
			$client_lang = ($jfLang->shortcode!='') ? $jfLang->shortcode : $jfLang->iso;
		}


		$overwriteGlobalConfig =  $params->get( 'overwriteGlobalConfig', 0 );
		if($overwriteGlobalConfig ) {
			// We should overwrite additional global variables based on the language parameter configuration
			$langParams = new mosParameters( $jfLang->params );
			foreach ($langParams->toArray() as $key => $value) {
				$GLOBALS['mosConfig_' .$key] =$value;
			}
		}


		// set locale for this ISO code
		setlocale(LC_TIME, $jfLang->iso);

		setcookie( "lang", "", time() - 1800, "/" );
		setcookie( "jfcookie", "", time() - 1800, "/" );
		setcookie( "jfcookie[lang]", $client_lang, time()+24*3600, '/' );
		$GLOBALS['iso_client_lang'] = $client_lang;
		$GLOBALS['mosConfig_lang'] = $jfLang->code;

		$registry->setValue("config.multilingual_support", true);

		global $mainframe;

		$mainframe->setUserState('application.lang',$jfLang->code);
		$registry->setValue("config.jflang", $jfLang->code);
		$registry->setValue("config.lang_site",$jfLang->code);
		$registry->setValue("config.language",$jfLang->code);
		$registry->setValue("joomfish.language",$jfLang);
		/*
		}
		else if ($client_lang=="eng") {
		$mainframe->setLanguage("en-GB");
		$registry->setValue("config.jflang", "english");
		$mainframe->setUserState('application.lang','en-GB');
		}
		*/


		// Get the global configuration object
		$registry =& JFactory::getConfig();

		// Load the configuration values into the registry
		$registry->loadObject($config);
		
		// overwrite with the valued from $jfLang
		$params = new JParameter($jfLang->params);
		$paramarray = $params->toArray();
		foreach ($paramarray as $key=>$val) {
			$registry->setValue("config.".$key,$val);
			
			if (defined("_JLEGACY")){
				$name = 'mosConfig_'.$key;
				$GLOBALS[$name] = $val;
			}		
		}

	}



	/** This function initialize the Joom!Fish manager in order to have
	  * easy access and prepare certain information.
	  * @access private
	  */
	function _jfInitialize ( ) {
		/* ToDo: check if we really need this any longer. Should be removed latest with 2.1
		* @deprecated
		*/
		$GLOBALS[ '_JOOMFISH_MANAGER'] 	 =&  JoomFishManager::getInstance();

	}
}

function routeJFRule($router, &$uri){
	$registry =& JFactory::getConfig();
	$multilingual_support= $registry->getValue("config.multilingual_support",false);
	$jfLang = $registry->getValue("joomfish.language", false);
	if ($multilingual_support && $jfLang){
		if ($uri->getVar("lang","")==""){
			$uri->setVar("lang",($jfLang->shortcode!='') ? $jfLang->shortcode : $jfLang->iso);
		}
		// this is dependent on Joomfish router being first!!
		$lang=$uri->getVar("lang","");

		// Get the path data
		$route = $uri->getPath();

		//Add the suffix to the uri
		if($router->_mode == JROUTER_MODE_SEF && $route && !$lang!==""){
			$uri->setPath($uri->getPath().'/lang-'.$lang);
			$uri->delVar("lang");
		}
	}
	return;
}

function parseJFRule($router, &$uri){
	//echo "got here too lang = ".$uri->getVar("lang","")."<br/>";
	$route = $uri->getPath();

	$segments = explode('/', $route);
	if (count($segments>0)){
		// this is dependent on Joomfish router being first!!
		if (strpos($segments[0],"lang-")===0){
			$lang = trim(str_replace("lang-","",$segments[0]));
			array_shift($segments);
			$uri->setPath(implode("/",$segments));

			// This get over written later - really stupid !!!
			$uri->setVar("lang",$lang);
			
			JRequest::setVar('lang', $lang );		
			// I need to discover language here since menu is loaded in router
			plgSystemJFDatabase::_discoverJFLanguage();
			return array("lang"=>$lang);
		}
	}
	plgSystemJFDatabase::_discoverJFLanguage();
	return array();
}
